<?php

/**
 * Set the content width based on the theme's design and stylesheet.
 *
 */
if ( ! isset( $content_width ) ) {
	$content_width = 660;
}



if ( ! function_exists( 'theme_setup' ) ) :

function theme_setup() 
{

	load_theme_textdomain( 'gede3', get_template_directory() . '/languages' );
	apply_filters( 'theme_locale', get_locale(), 'gede3' );

	// Add default posts and comments RSS feed links to head.
	add_theme_support( 'automatic-feed-links' );


	add_theme_support( 'title-tag' );


	add_theme_support( 'post-thumbnails' );
	set_post_thumbnail_size( 825, 510, true );

	// This theme uses wp_nav_menu() in two locations.
	register_nav_menus( array(
		'primary' => __( 'القائمه الرئيسيه',      'gede3' ),
		'footer-a'  => __( 'قائمه الفوتر الأولى',      'gede3' ),
		'footer'  => __( 'قائمه السفليه',      'gede3' ),
	) );


	add_theme_support( 'html5', array(
		'search-form', 'comment-form', 'comment-list', 'gallery', 'caption'
	) );


	add_editor_style( array( 'assets/css/editor-style.css', 'genericons/genericons.css' ) );
}
endif; // gede3_setup
add_action( 'after_setup_theme', 'theme_setup' );





/**
 * Register widget area.
 */
function theme_widgets_init() {
	register_sidebar( array(
		'name'          => __( 'المربعات الجانبيه', 'gede3' ),
		'id'            => 'sidebar-1',
		'description'   => __( 'مربعات القائمه الجانبيه.', 'gede3' ),
		'before_widget' => '<aside id="%1$s" class="widget %2$s">',
		'after_widget'  => '</div></aside>',
		'before_title'  => '<h3 class="widget-title">',
		'after_title'   => '</h3><div class="widget-entry clearfix">',
	) );
}
add_action( 'widgets_init', 'theme_widgets_init' );





/**
 * Enqueue scripts and styles.
 */
function theme_scripts() {

	define( 'ASSET', get_template_directory_uri() );

	// -----------------------------------------
	// styles                    -
	// -----------------------------------------
	wp_enqueue_style( 'font-awesome', ASSET . '/assets/libs/fontawesome/css/font-awesome.css', array(), '' );
	wp_enqueue_style( 'bootstrap', ASSET . '/assets/libs/bootstrap/dist/css/bootstrap.min.css', array(), '' );
	wp_enqueue_style( 'bootstrap-rtl', ASSET . '/assets/libs/bootstrap-rtl/dist/css/bootstrap-rtl.min.css', array(), '' );
	wp_enqueue_style( 'bs-custom', ASSET . '/assets/css/bs-custom.css', array(), '' );
	wp_enqueue_style( 'wordpress.css', ASSET . '/assets/css/wordpress.css', array(), '' );
	wp_enqueue_style( 'sidebar.css', ASSET . '/assets/css/sidebar.css', array(), '' );
	wp_enqueue_style( 'comments.css', ASSET . '/assets/css/comments.css', array(), '' );
	wp_enqueue_style( 'wp-general.css', ASSET . '/assets/css/wp-general.css', array(), '' );



	wp_enqueue_style( 'wow-animate', ASSET . '/assets/libs/wow/css/libs/animate.css', array(), '' );
	wp_enqueue_style( 'animate.css', ASSET . '/assets/libs/animate.css/animate.css', array(), '' );
	wp_enqueue_style( 'hover-min', ASSET . '/assets/libs/Hover/css/hover-min.css', array(), '' );
	wp_enqueue_style( 'owl.carousel', ASSET . '/assets/libs/owl-carousel/assets/owl.carousel.css', array(), '' );
	wp_enqueue_style( 'flexslider', ASSET . '/assets/libs/flexslider/flexslider.css', array(), '' );
	wp_enqueue_style( 'fancybox', ASSET . '/assets/libs/fancybox/jquery.fancybox.css', array(), '' );



	wp_enqueue_style( 'app.css', ASSET . '/assets/css/app.css', array(), '' );
	wp_enqueue_style( 'responsive', ASSET . '/assets/css/responsive.css', array(), '' );
	wp_enqueue_style( 'main-style', get_stylesheet_uri() );

	// -----------------------------------------
	// scripts                    -
	// -----------------------------------------
	wp_enqueue_script( 'bootstrap', ASSET . '/assets/libs/bootstrap/dist/js/bootstrap.min.js', array( 'jquery' ), '20141212', true );
	wp_enqueue_script( 'wow', ASSET . '/assets/libs/wow/dist/wow.min.js', array( 'jquery' ), '20141212', true );
	wp_enqueue_script( 'jquery.mixitup', ASSET . '/assets/libs/mixitup-master/src/jquery.mixitup.js', array( 'jquery' ), '20141212', true );
	wp_enqueue_script( 'owl-carousel', ASSET . '/assets/libs/owl-carousel/owl.carousel.js', array( 'jquery' ), '20141212', true );
	wp_enqueue_script( 'jquery.flexslider', ASSET . '/assets/libs/flexslider/jquery.flexslider.js', array( 'jquery' ), '20141212', true );
	wp_enqueue_script( 'fancybox', ASSET . '/assets/libs/fancybox/jquery.fancybox.js', array( 'jquery' ), '20141212', true );
	


	wp_enqueue_script( 'app.js', ASSET . '/assets/js/app.js', array( 'jquery' ), '20141212', true );

	wp_localize_script( 'app.js', 'headJS', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ), 'templateurl' => get_template_directory_uri(), 'posts_per_page' => get_option('posts_per_page') ) );

	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' );
	}
	// wp_enqueue_script( 'ta3limi-script', get_template_directory_uri() . '/js/functions.js', array( 'jquery' ), '20141212', true );
	// wp_localize_script( 'ta3limi-script', 'screenReaderText', array(
	// 	'expand'   => '<span class="screen-reader-text">' . __( 'expand child menu', 'ta3limi' ) . '</span>',
	// 	'collapse' => '<span class="screen-reader-text">' . __( 'collapse child menu', 'ta3limi' ) . '</span>',
	// ) );
}
add_action( 'wp_enqueue_scripts', 'theme_scripts' );